@@ -568,7 +568,8 @@ is_chroma_type_supported (GstVaapiEncoder * encoder)
     return TRUE;
 
   if (cip->chroma_type != GST_VAAPI_CHROMA_TYPE_YUV420 &&
-      cip->chroma_type != GST_VAAPI_CHROMA_TYPE_YUV422)
+      cip->chroma_type != GST_VAAPI_CHROMA_TYPE_YUV422 &&
+      cip->chroma_type != GST_VAAPI_CHROMA_TYPE_YUV420_10BPP)
     goto unsupported;
 
   if (!get_config_attribute (encoder, VAConfigAttribRTFormat, &format))
@@ -579,6 +580,7 @@ is_chroma_type_supported (GstVaapiEncoder * encoder)
 
   return TRUE;
 
+  /* ERRORS */
 unsupported:
   {
     GST_ERROR ("We only support YUV 4:2:0 and YUV 4:2:2 for encoding. "
@@ -587,6 +589,42 @@ unsupported:
   }
 }
 
+static guint
+get_default_chroma_type (GstVaapiEncoder * encoder,
+    const GstVaapiContextInfo * cip)
+{
+  guint value;
+
+  if (!gst_vaapi_get_config_attribute (encoder->display,
+          gst_vaapi_profile_get_va_profile (cip->profile),
+          gst_vaapi_entrypoint_get_va_entrypoint (cip->entrypoint),
+          VAConfigAttribRTFormat, &value))
+    return 0;
+
+  return to_GstVaapiChromaType (value);
+}
+
+static void
+init_context_info (GstVaapiEncoder * encoder, GstVaapiContextInfo * cip,
+    GstVaapiProfile profile)
+{
+  const GstVaapiEncoderClassData *const cdata =
+      GST_VAAPI_ENCODER_GET_CLASS (encoder)->class_data;
+
+  cip->usage = GST_VAAPI_CONTEXT_USAGE_ENCODE;
+  cip->profile = profile;
+  if (cdata->codec == GST_VAAPI_CODEC_JPEG) {
+    cip->entrypoint = GST_VAAPI_ENTRYPOINT_PICTURE_ENCODE;
+  } else {
+    if (cip->entrypoint != GST_VAAPI_ENTRYPOINT_SLICE_ENCODE_LP)
+      cip->entrypoint = GST_VAAPI_ENTRYPOINT_SLICE_ENCODE;
+  }
+  cip->chroma_type = get_default_chroma_type (encoder, cip);
+  cip->width = 0;
+  cip->height = 0;
+  cip->ref_frames = encoder->num_ref_frames;
+}
+
 /* Updates video context */
 static gboolean
 set_context_info (GstVaapiEncoder * encoder)
@@ -595,20 +633,12 @@ set_context_info (GstVaapiEncoder * encoder)
   GstVaapiConfigInfoEncoder *const config = &cip->config.encoder;
   const GstVideoFormat format =
       GST_VIDEO_INFO_FORMAT (GST_VAAPI_ENCODER_VIDEO_INFO (encoder));
-  const GstVaapiEncoderClassData *const cdata =
-      GST_VAAPI_ENCODER_GET_CLASS (encoder)->class_data;
 
-  cip->usage = GST_VAAPI_CONTEXT_USAGE_ENCODE;
-  cip->profile = encoder->profile;
-  if (cdata->codec != GST_VAAPI_CODEC_JPEG) {
-    if (cip->entrypoint != GST_VAAPI_ENTRYPOINT_SLICE_ENCODE_LP)
-      cip->entrypoint = GST_VAAPI_ENTRYPOINT_SLICE_ENCODE;
-  } else
-    cip->entrypoint = GST_VAAPI_ENTRYPOINT_PICTURE_ENCODE;
+  init_context_info (encoder, cip, get_profile (encoder));
+
   cip->chroma_type = gst_vaapi_video_format_get_chroma_type (format);
   cip->width = GST_VAAPI_ENCODER_WIDTH (encoder);
   cip->height = GST_VAAPI_ENCODER_HEIGHT (encoder);
-  cip->ref_frames = encoder->num_ref_frames;
 
   if (!is_chroma_type_supported (encoder))
     goto error_unsupported_format;
@@ -722,9 +752,6 @@ gst_vaapi_encoder_set_codec_state (GstVaapiEncoder * encoder,
   g_return_val_if_fail (state != NULL,
       GST_VAAPI_ENCODER_STATUS_ERROR_INVALID_PARAMETER);
 
-  if (encoder->num_codedbuf_queued > 0)
-    goto error_operation_failed;
-
   if (!gst_video_info_is_equal (&state->info, &encoder->video_info)) {
     status = check_video_info (encoder, &state->info);
     if (status != GST_VAAPI_ENCODER_STATUS_SUCCESS)
@@ -732,13 +759,6 @@ gst_vaapi_encoder_set_codec_state (GstVaapiEncoder * encoder,
     encoder->video_info = state->info;
   }
   return gst_vaapi_encoder_reconfigure_internal (encoder);
-
-  /* ERRORS */
-error_operation_failed:
-  {
-    GST_ERROR ("could not change codec state after encoding started");
-    return GST_VAAPI_ENCODER_STATUS_ERROR_OPERATION_FAILED;
-  }
 }
 
 /**
@@ -852,11 +872,12 @@ get_rate_control_mask (GstVaapiEncoder * encoder)
         continue;
       rate_control_mask |= 1 << to_GstVaapiRateControl (1 << i);
     }
+    GST_INFO ("supported rate controls: 0x%08x", rate_control_mask);
+
+    encoder->got_rate_control_mask = TRUE;
+    encoder->rate_control_mask = cdata->rate_control_mask & rate_control_mask;
   }
-  GST_INFO ("supported rate controls: 0x%08x", rate_control_mask);
 
-  encoder->got_rate_control_mask = TRUE;
-  encoder->rate_control_mask = cdata->rate_control_mask & rate_control_mask;
   return encoder->rate_control_mask;
 }
 
@@ -1132,9 +1153,117 @@ gst_vaapi_encoder_new (const GstVaapiEncoderClass * klass,
     goto error;
   return encoder;
 
+  /* ERRORS */
 error:
-  gst_vaapi_encoder_unref (encoder);
-  return NULL;
+  {
+    gst_vaapi_encoder_unref (encoder);
+    return NULL;
+  }
+}
+
+static GstVaapiContext *
+create_test_context_config (GstVaapiEncoder * encoder, GstVaapiProfile profile)
+{
+  GstVaapiContextInfo cip = { 0, };
+  GstVaapiContext *ctxt;
+
+  if (encoder->context)
+    return gst_vaapi_object_ref (encoder->context);
+
+  /* if there is no profile, let's figure out one */
+  if (profile == GST_VAAPI_PROFILE_UNKNOWN)
+    profile = get_profile (encoder);
+
+  init_context_info (encoder, &cip, profile);
+  ctxt = gst_vaapi_context_new (encoder->display, &cip);
+  return ctxt;
+}
+
+static GArray *
+get_profile_surface_formats (GstVaapiEncoder * encoder, GstVaapiProfile profile)
+{
+  GstVaapiContext *ctxt;
+  GArray *formats;
+
+  ctxt = create_test_context_config (encoder, profile);
+  if (!ctxt)
+    return NULL;
+  formats = gst_vaapi_context_get_surface_formats (ctxt);
+  gst_vaapi_object_unref (ctxt);
+  return formats;
+}
+
+static gboolean
+merge_profile_surface_formats (GstVaapiEncoder * encoder,
+    GstVaapiProfile profile, GArray * formats)
+{
+  GArray *surface_fmts;
+  guint i, j;
+  GstVideoFormat fmt, sfmt;
+
+  if (profile == GST_VAAPI_PROFILE_UNKNOWN)
+    return FALSE;
+
+  surface_fmts = get_profile_surface_formats (encoder, profile);
+  if (!surface_fmts)
+    return FALSE;
+
+  for (i = 0; i < surface_fmts->len; i++) {
+    sfmt = g_array_index (surface_fmts, GstVideoFormat, i);
+    for (j = 0; j < formats->len; j++) {
+      fmt = g_array_index (formats, GstVideoFormat, j);
+      if (fmt == sfmt)
+        break;
+    }
+    if (j >= formats->len)
+      g_array_append_val (formats, sfmt);
+  }
+
+  g_array_unref (surface_fmts);
+  return TRUE;
+}
+
+/**
+ * gst_vaapi_encoder_get_surface_formats:
+ * @encoder: a #GstVaapiEncoder instances
+ *
+ * Fetches the valid surface formats for the current VAConfig
+ *
+ * Returns: a #GArray of valid formats for the current VAConfig
+ **/
+GArray *
+gst_vaapi_encoder_get_surface_formats (GstVaapiEncoder * encoder,
+    GstVaapiProfile profile)
+{
+  const GstVaapiEncoderClassData *const cdata =
+      GST_VAAPI_ENCODER_GET_CLASS (encoder)->class_data;
+  GArray *profiles, *formats;
+  guint i;
+
+  if (profile || encoder->context)
+    return get_profile_surface_formats (encoder, profile);
+
+  /* no specific context neither specific profile, let's iterate among
+   * the codec's profiles */
+  profiles = gst_vaapi_display_get_encode_profiles (encoder->display);
+  if (!profiles)
+    return NULL;
+
+  formats = g_array_new (FALSE, FALSE, sizeof (GstVideoFormat));
+  for (i = 0; i < profiles->len; i++) {
+    profile = g_array_index (profiles, GstVaapiProfile, i);
+    if (gst_vaapi_profile_get_codec (profile) == cdata->codec) {
+      if (!merge_profile_surface_formats (encoder, profile, formats)) {
+        g_array_unref (formats);
+        formats = NULL;
+        break;
+      }
+    }
+  }
+
+  g_array_unref (profiles);
+
+  return formats;
 }
 
 /** Returns a GType for the #GstVaapiEncoderTune set */
