@@ -59,7 +59,11 @@
 #define GST_PLUGIN_DESC "A VA-API based videosink"
 
 GST_DEBUG_CATEGORY_STATIC (gst_debug_vaapisink);
+#ifndef GST_DISABLE_GST_DEBUG
 #define GST_CAT_DEFAULT gst_debug_vaapisink
+#else
+#define GST_CAT_DEFAULT NULL
+#endif
 
 /* Default template */
 /* *INDENT-OFF* */
@@ -1213,8 +1217,17 @@ static gboolean
 gst_vaapisink_start (GstBaseSink * base_sink)
 {
   GstVaapiSink *const sink = GST_VAAPISINK_CAST (base_sink);
+  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (base_sink);
 
-  return gst_vaapi_plugin_base_open (GST_VAAPI_PLUGIN_BASE (sink));
+  if (!gst_vaapisink_ensure_display (sink))
+    return FALSE;
+
+  /* Ensures possible raw caps earlier to avoid race conditions at
+   * get_caps() */
+  if (!gst_vaapi_plugin_base_get_allowed_raw_caps (plugin))
+    return FALSE;
+
+  return TRUE;
 }
 
 static gboolean
@@ -1321,6 +1334,8 @@ gst_vaapisink_set_caps (GstBaseSink * base_sink, GstCaps * caps)
   sink->video_height = GST_VIDEO_INFO_HEIGHT (vip);
   sink->video_par_n = GST_VIDEO_INFO_PAR_N (vip);
   sink->video_par_d = GST_VIDEO_INFO_PAR_D (vip);
+  if (sink->video_par_n == 0)
+    sink->video_par_n = 1;
   GST_DEBUG ("video pixel-aspect-ratio %d/%d",
       sink->video_par_n, sink->video_par_d);
 
@@ -1511,13 +1526,12 @@ gst_vaapisink_propose_allocation (GstBaseSink * base_sink, GstQuery * query)
 static gboolean
 gst_vaapisink_query (GstBaseSink * base_sink, GstQuery * query)
 {
-  GstVaapiSink *const sink = GST_VAAPISINK_CAST (base_sink);
-  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (sink);
+  GstElement *const element = GST_ELEMENT (base_sink);
   gboolean ret = FALSE;
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_CONTEXT:
-      ret = gst_vaapi_handle_context_query (query, plugin->display);
+      ret = gst_vaapi_handle_context_query (element, query);
       break;
     default:
       ret = GST_BASE_SINK_CLASS (gst_vaapisink_parent_class)->query (base_sink,
@@ -1695,19 +1709,6 @@ gst_vaapisink_event (GstBaseSink * base_sink, GstEvent * event)
   return res;
 }
 
-static void
-gst_vaapisink_set_bus (GstElement * element, GstBus * bus)
-{
-  /* Make sure to allocate a VA display in the sink element first,
-     so that upstream elements could query a display that was
-     allocated here, and that exactly matches what the user
-     requested through the "display" property */
-  if (!GST_ELEMENT_BUS (element) && bus)
-    gst_vaapisink_ensure_display (GST_VAAPISINK_CAST (element));
-
-  GST_ELEMENT_CLASS (gst_vaapisink_parent_class)->set_bus (element, bus);
-}
-
 static void
 gst_vaapisink_class_init (GstVaapiSinkClass * klass)
 {
@@ -1742,7 +1743,6 @@ gst_vaapisink_class_init (GstVaapiSinkClass * klass)
   videosink_class->show_frame = GST_DEBUG_FUNCPTR (gst_vaapisink_show_frame);
 
   element_class->set_context = gst_vaapi_base_set_context;
-  element_class->set_bus = gst_vaapisink_set_bus;
   gst_element_class_set_static_metadata (element_class,
       "VA-API sink", "Sink/Video", GST_PLUGIN_DESC,
       "Gwenole Beauchesne <gwenole.beauchesne@intel.com>");
